#version 120

// DoF - Effect: http://ati.amd.com/developer/shaderx/shaderx2_real-timedepthoffieldsimulation.pdf

uniform sampler2D texture0;
uniform vec3 temp;

uniform vec4 material_ambient;
uniform vec4 material_diffuse;
uniform vec4 material_specular;
uniform vec4 material_emissive;
uniform float material_shininess;

uniform float focalLen; 
uniform float Zfocus;
uniform float maxCoC;
uniform float Dlens;
uniform float scale;
uniform float sceneRange;

struct light {
	vec4 position;
	vec4 diffuse;
	vec4 specular;
	vec4 ambient;

	float constant_attenuation;
	float linear_attenuation;
	float quadratic_attenuation;
};

uniform light light0;

varying vec2 texCoord0;
varying vec3 normal;
varying vec3 lightDir;
varying vec3 halfVector;

varying vec4 vertDiffuse;
varying vec4 vertAmbient;
varying float dist;
varying float Depth;

void main(void) {
	
	vec3 N = normalize(normal); //Renormalize the normal
	
	//Find the angle between the normal and the light
	float NdotL = max(dot(N, normalize(lightDir)), 0.0);
	
	vec4 color = vec4(0.5); // 0.1 - 0.5 
	vec4 toonColor;
	
	//If the normal is facing the light
	if (NdotL > 0.0)
	{								 
		//Add the diffuse contribution
		color += vertDiffuse * NdotL;
		
		//Renormalize the half vector
		vec3 HV = normalize(halfVector);
		float NdotHV = max(dot(N, HV), 0.0);
		//Add the specular contribution
		color += material_specular * light0.specular * pow(NdotHV, material_shininess);	
	}
		//Calculate the attenuation factor
	float attenuation = 1.0 / (light0.constant_attenuation + light0.linear_attenuation * dist + light0.quadratic_attenuation * dist * dist);		

	// Compute blur factor based on the CoC size scaled and 
	// normalized to 0..1 range
	float pixCoC = abs(Dlens * focalLen * (Zfocus - Depth) / (Zfocus * (Depth - focalLen))); 
	
	// float blur = saturate(pixCoC * scale / maxCoC);
	float blur = clamp(pixCoC * scale / maxCoC, 0.0, 1.0); 

	// toon shader
	//float intensity = dot(vec3(light0.position),N);
	vec4 colorMap =  ((color * attenuation) + vertAmbient) * texture2D(texture0, texCoord0.st);
	vec4 finalOutput;
	
		if (NdotL > 0.95)
			finalOutput = vec4(1.0);
		else if (NdotL > 0.5)
			finalOutput = vec4(0.75);
		else if (NdotL > 0.25)
			finalOutput = vec4(0.5);
		else
			finalOutput = vec4(0.25);
			
	finalOutput *=  colorMap *2;

	//Output the final color after taking attenuation and ambient light into account								 	
	//gl_FragData[0] = ((color * attenuation) + vertAmbient) * texture2D(texture0, texCoord0.st);

	gl_FragData[0] = finalOutput;
	
	// Depth/blurriness value scaled to 0..1 range
	// o.vDoF = float4(v.fDepth / sceneRange, blur, 0, 0);
	gl_FragData[1] = vec4(Depth / sceneRange, blur, 0, 0);
}
