#version 120

// http://ati.amd.com/developer/shaderx/shaderx2_real-timedepthoffieldsimulation.pdf

uniform float Width, Height;
uniform sampler2D texture0, texture1;
uniform float maxCoC;

//varying vec2 texCoord0;
//varying vec2 texCoord1;

void main (void)
{
    vec4 colorSum, tapColor;
    vec2 centerDepthBlur, tapCoord, tapDepthBlur;
    vec4 toonColor;

	float totalContribution, tapContribution;

	// Scale tap offsets based on render target size
	float dx = 1.0/Width;
	float dy = 1.0/Height;

	// D3DXVECTOR4 v[12];
	vec2 filterTaps[12];
	filterTaps[0] = vec2(-0.326212 * dx, -0.40581 * dy);
	filterTaps[1] = vec2(-0.840144 * dx, -0.07358 * dy);
	filterTaps[2] = vec2(-0.695914 * dx, 0.457137 * dy);
	filterTaps[3] = vec2(-0.203345 * dx, 0.620716 * dy);
	filterTaps[4] = vec2(0.96234 * dx, -0.194983 * dy);
	filterTaps[5] = vec2(0.473434 * dx, -0.480026 * dy);
	filterTaps[6] = vec2(0.519456 * dx, 0.767022 * dy);
	filterTaps[7] = vec2(0.185461 * dx, -0.893124 * dy);
	filterTaps[8] = vec2(0.507431 * dx, 0.064425 * dy);
	filterTaps[9] = vec2(0.89642 * dx, 0.412458 * dy);
	filterTaps[10] = vec2(-0.32194 * dx, -0.932615 * dy);
	filterTaps[11] = vec2(-0.791559 * dx, -0.59771 * dy);


	// Start with center sample color
	//colorSum = texture2D(texture0, texCoord0.st);
	colorSum = texture2D(texture0, gl_TexCoord[0].st);
	totalContribution = 1.0;
	
	//Depth and blurriness values for center sample
	//centerDepthBlur = texture2D(texture1, texCoord1.st).xy;
	centerDepthBlur = texture2D(texture1, gl_TexCoord[1].st).xy;

	float sizeCoC = centerDepthBlur.y * maxCoC;

	// Run through all filter taps
	for(int i = 0; i < 12; i++) {
		// Compute sample coordinates
		tapCoord = gl_TexCoord[0].st + filterTaps[i] * sizeCoC;
		// Fetch filter tap sample
		tapColor = texture2D(texture0, tapCoord);
		tapDepthBlur = texture2D(texture1, tapCoord).xy;
		// Compute tap contribution based on depth and blurriness
		tapContribution = (tapDepthBlur.x > centerDepthBlur.x) ? 1.0 : tapDepthBlur.y;
		// Accumulate color and sample contribution
		colorSum += tapColor * tapContribution;
		totalContribution += tapContribution;
	}
	
	vec4 finalColor = colorSum / totalContribution;
	
	// Send final result
	
	gl_FragData[0] = finalColor;
}
